#!/usr/bin/perl
# File: upload.pl 
# written by horshack@lisa.franken.de

BEGIN {
        use lib '.';
}

use strict;
use CGI;
use CGI qw(:standard);
use Digest::MD5  qw(md5 md5_hex md5_base64);
use File::Basename;
use Config::IniFiles;
use HTML::Table;


print "Content-Type: text/html\n\n";

my $VERSION = "1.01";
# maximum size of files to upload
$CGI::POST_MAX = 15_000_000;	

my $ME = basename($0) . " V$VERSION";

my $ini_fqn= basename($0,".pl") . ".ini";
my $ini = new Config::IniFiles( -file => $ini_fqn);

my $upload_fqdn = $ini->val('upload','upload_base_fqdn');
unless (-d $upload_fqdn) {
	print "Could not find directory upload_base_fqdn. Aborting.\n";
	exit;
}

my $WF = new CGI;
my $DEBUG= $WF->param('debug') || 0;

# Form to upload manually
unless ($WF->param('upfile')) {
        print create_uploadform();
} else {
	# you need a password to upload files here,
	# but only if configured
	if ($ini->val('upload','upload_pass')) {
		# is the password the one from the inifile?
		unless ($WF->param('upload_pass') eq $ini->val('upload','upload_pass')) {
			print "You are allowed to upload only with the right password...<br>";
			exit;
		}
	}
        &manage_upload(upload_fqdn=>$upload_fqdn,md5=>$WF->param('md5_client'));
}

exit;


sub create_uploadform {
        my $html;
        my ($left,$right,$tab);

	$html .= $WF->start_html(-title=>$ME);
        $html .=<<EOHEAD;
<h1>$ME</h1>

Attention! Sometimes Problems with sizelimits because using POST with Squid-proxyserver. Default limit is 1 Megabyte. Squidparam in squid.conf should be 'request_body_max_size 0' to allow unlimited size of upload if you would allow that...<p>
EOHEAD

        # this is for the upload itself
        $html .= $WF->start_multipart_form(
                -method=>'POST',
                -action=> $WF->url()
	);

	# create a table for the upload form
	my $tab = HTML::Table->new(0,2);	# y,x!
        $left = "Upload this file:";
        $right = $WF ->filefield(
                        -name=>'upfile',
                        -default=>'',
                        -size=>30,
                        -maxlength=>80);
        $tab->addRow($left,$right);

        $left = "Subdirectory to store file in (optional):";
	$right = $WF->textfield(-name=>'upfile_subdir',
                                -value=>'', -size=>40, -maxlength=>40);

        $tab->addRow($left,$right);
        $left = "MD5 for this file (optional):";
	$right = $WF->textfield(-name=>'md5_client',
                                -value=>'', -size=>40, -maxlength=>40);
        $tab->addRow($left,$right);

        $left = "Size of this file (optional):";
	$right = $WF->textfield(-name=>'upfile_size',
                                -value=>'', -size=>10, -maxlength=>10) . " Bytes";
        $tab->addRow($left,$right);

        $left = "Password:";
	$right = $WF->password_field(-name=>'upload_pass',
                                -value=>'', -size=>40, -maxlength=>40);
        $tab->addRow($left,$right);

        $left = "";
        $right = $WF->submit(-name=>"",-value=>"Start upload");
        $tab->addRow($left,$right);

	$html .= $tab->getTable();
	$html .= $WF->end_multipart_form ();
	$html .= &footer;
	return $html;
}



sub manage_upload {
	my (%args) = @_;
	my $upload_fqdn = $args{upload_fqdn};
	my $md5 = $args{md5};
	my $html;

        # Does the upload-directory exist?
        unless (-d $upload_fqdn) {
                $html .= print "Directory for upload $upload_fqdn not found. Abort.<br>";
                print $html;
                exit;
        }
	if (($md5 eq "") or (not $md5)) {
		$md5=undef;
	}

        # Uploading file
        $html .= "<h1>$ME</h1>";
        my $remote_fqn = $WF->param('upfile');
        # trim remote filename, remove directory
        my @parts = split ( /[\\\/]/, $remote_fqn );
        my $remote_fn = $parts[-1];
	# Should I upload the file into a subdirectory?
	if ($WF->param('upfile_subdir')) {
        	$upload_fqdn .= "/" . $WF->param('upfile_subdir');
		unless (-d $upload_fqdn) {
			print "Could not write file into subdir " . $WF->param('upfile_subdir');
			print "<br>Abort.";
			exit;
		}
	}
        my $local_fqn = $upload_fqdn . "/" . $remote_fn;

	my $tab = HTML::Table->new(0,2);	# y,x!
        my ($left,$right);

	# delete old file if there is one
	if (-e $local_fqn) {
	        $html .= "Deleting file $local_fqn<br>";
       		unlink $local_fqn;
	}
        if (-e $local_fqn) {
                $html .= "Could not delete file ${local_fqn}. Abort.<br>";
                print $html;
                exit;
	}


        my $remote_subdir = $WF->param('upfile_subdir');
        no strict;
	my ($bytesread,$buffer);
        open (OUT,">$local_fqn") || die "Could not open file $local_fqn for writing! Abort.<br>";
        binmode OUT;

	if ($CGI::VERSION < '2.56') {
		$html .= "CGI-Version is less than V2.56, exactly V" .
			$CGI::VERSION . "<br>";
		while (<$remote_fqn>) {
			print OUT $_;
		}
	} else {
		# From CGI-version v2.56 
		$html .= "CGI-Version is greater than V2.56, exactly V" .
			$CGI::VERSION . "<br>";
		my $fh = $WF->upload('upfile');
		binmode $fh;
		while (<$fh>) {
			print OUT $_;
		}
		while ($bytesread=read($remote_fqn,$buffer,1024)) {
		   	print OUT $buffer;
		}
	}
        close OUT;
        use strict;

	# Build local md5-checksum as hexdigest
	my $local_md5 = undef;

	# read file for that
	my $bucket;
	open (MD,$local_fqn);
	binmode MD;
	while (<MD>) {
		$bucket .= $_
	}
	close MD;
	# build MD5
	my $ctx = Digest::MD5->new;
	$ctx->add($bucket);
	$local_md5 = $ctx->hexdigest;
	# End of md5

	# Output status after upload
	my $md5hex_server = $local_md5;
	my $md5hex_client = $WF->param('md5_client');
	my $fqn_client = $WF->param('upfile');
	my $fn_client = $remote_fn;
	my $size_server = -s $local_fqn;
	my $size_client = $WF->param('upfile_size') || undef;

	my $cutter_begin="=== begin of uploadresult ===";
	my $cutter_end="=== end of uploadresult ===";
	my ($beginner,$ender)=("bol","eol");
	$html .= sprintf "<pre>";
	$html .= sprintf "%s\n",$cutter_begin;
	$html .= sprintf "$beginner|%s|%s|$ender\n","fqn_server",$local_fqn;
	$html .= sprintf "$beginner|%s|%s|$ender\n","fn_client",$fn_client;
	$html .= sprintf "$beginner|%s|%s|$ender\n","fqn_client",$fqn_client;
	$html .= sprintf "$beginner|%s|%s|$ender\n","size_client",$size_client;
	$html .= sprintf "$beginner|%s|%s|$ender\n","size_server",$size_server;
	$html .= sprintf "$beginner|%s|%s|$ender\n","md5hex_client", $md5hex_client;
	$html .= sprintf "$beginner|%s|%s|$ender\n","md5hex_server", $md5hex_server;

	# judge upload, did it work?
	my $bad=0;
	my $com;	# comments here
	if ($size_server == 0) {
		$bad++;
		$com .= "Filesize is 0 Bytes. Bad...<br>";
	}
	# Compare filesize there and here 
	# but only if the uploader gave us a filesize
	if ($size_client) {
		unless ($size_server == $size_client) {
			$bad++;
			$com .= "filesize here and there don't match. Bad...<br>";
		}
	}
	# Compare MD5-checksums
	# but only if the uploader gave us one
	if ($md5hex_client) {
		unless ($md5hex_server eq $md5hex_client) {
			$bad++;
			$com .= "MD5-checksums here and there don't match. Bad...<br>";
		}
	}

	if ($bad == 0) {
		$com = "Upload of file is ok!<br>";
		$html .= sprintf "$beginner|%s|%s|$ender\n","uploadstatus", "ok";
	} else {
		$com .= "Upload of file with failures!<br>";
		$html .= sprintf "$beginner|%s|%s|$ender\n","uploadstatus", "failure";
	}

	$html .= sprintf "%s<br>\n",$cutter_end;
	$html .= sprintf "</pre>";

	# Comments to the end of the webpage
	$html .= $com;

        print $html;
}

sub footer {
	return "<hr>Program $ME, CGI module version $CGI::VERSION";
}
